@@ -2,6 +2,7 @@ import logging
 import ssl
 from base64 import urlsafe_b64encode
 from datetime import datetime
+from time import sleep
 from socket import socket, AF_INET, SOCK_STREAM, MSG_PEEK
 from pathlib import Path
 from cryptography.x509 import extensions, oid
@@ -9,11 +10,10 @@ from cryptography.x509.base import Certificate
 from cryptography.x509.ocsp import OCSPResponse, load_der_ocsp_response, OCSPResponseStatus, OCSPCertStatus, OCSPRequestBuilder
 from cryptography.hazmat.primitives.hashes import SHA1
 from cryptography.hazmat.primitives.serialization import Encoding
-from OpenSSL import SSL
+from OpenSSL import SSL, _util
 from OpenSSL.SSL import _lib as native_openssl
 from OpenSSL.crypto import X509, FILETYPE_PEM, X509Name, load_certificate
 from certifi import where
-from retry.api import retry
 from hyperframe.frame import Frame, SettingsFrame
 from hyperframe.exceptions import InvalidFrameError
 from rich.progress import Progress, TaskID
@@ -27,6 +27,8 @@ from . import exceptions
 __module__ = 'tlsverify.metadata'
 logger = logging.getLogger(__name__)
 
+FAKE_PROTOCOLS = ['TLSv1.4', 'TLSv1.8', 'TLSv2', 'TLSv2.1', 'TLSv2.3']
+
 class Transport:
     host :str
     port :int
@@ -64,6 +66,13 @@ class Transport:
     client_certificate :X509
     client_certificate_expected :bool
     client_certificate_match :bool
+    tls_downgrade :bool
+    offered_tls_versions :list
+    tls_version_intolerance :bool
+    tls_version_intolerance_versions :list
+    tls_version_interference :bool
+    tls_version_interference_versions :list
+    preferred_protocol :str
     cafiles :list
     certificate_chain :list[X509]
     verifier_errors :list[tuple[X509], int]
@@ -105,6 +114,13 @@ class Transport:
         self.http2_support = False
         self.http2_cleartext_support = False
         self.peer_address = None
+        self.tls_downgrade = None
+        self.offered_tls_versions = []
+        self.tls_version_interference = None
+        self.tls_version_interference_versions = []
+        self.tls_version_intolerance = None
+        self.tls_version_intolerance_versions = []
+        self.preferred_protocol = None
         self.ocsp_stapling = None
         self.ocsp_must_staple = None
         self.ocsp_revocation_reason = None
@@ -113,21 +129,19 @@ class Transport:
         self.ocsp_certificate_status = None
         self.depth = {}
 
-    def pre_client_authentication_check(self, client_pem_path :str = None, updater :tuple[Progress, TaskID] = None) -> bool:
+    def pre_client_authentication_check(self, client_pem_path :str = None, progress_bar :callable = lambda *args: None) -> bool:
         if not isinstance(self.port, int):
             raise TypeError(f"provided an invalid type {type(self.port)} for port, expected int")
         if validators.domain(self.host) is not True:
             raise ValueError(f"provided an invalid domain {self.host}")
         if not isinstance(client_pem_path, str):
             raise TypeError(f"provided an invalid type {type(client_pem_path)} for client_pem_path, expected str")
-        progress, task = (None, None)
-        if isinstance(updater, tuple):
-            progress, task = updater
+
         self.client_certificate_expected = True
         self.client_certificate_match = False
         valid_client_pem = util.filter_valid_files_urls([client_pem_path], self.tmp_path_prefix)
         if valid_client_pem is False:
-            logger.error(f'client_pem_path was provided but is not a valid URL or file does not exist')
+            logger.error('client_pem_path was provided but is not a valid URL or file does not exist')
             return False
         if isinstance(valid_client_pem, list) and len(valid_client_pem) == 1:
             self.client_pem_path = valid_client_pem[0]
@@ -136,22 +150,17 @@ class Transport:
         ctx.load_verify_locations(cafile=where())
         ctx.verify_mode = SSL.VERIFY_NONE
         ctx.check_hostname = False
-        conn = SSL.Connection(ctx, socket(AF_INET, SOCK_STREAM))
+        conn = SSL.Connection(ctx, self.prepare_socket())
         conn.connect((self.host, self.port))
-        conn.settimeout(3)
         if ssl.HAS_SNI:
             conn.set_tlsext_host_name(idna.encode(self.host))
         conn.setblocking(1)
         conn.set_connect_state()
-        if isinstance(progress, Progress): progress.update(task, advance=1)
-        try:
-            conn.do_handshake()
-            self.expected_client_subjects = conn.get_client_ca_list()
-        except Exception as ex:
-            logger.warning(ex, exc_info=True)
-        finally:
-            conn.close()
-        if isinstance(progress, Progress): progress.update(task, advance=1)
+        progress_bar()
+        util.do_handshake(conn)
+        self.expected_client_subjects = conn.get_client_ca_list()
+        conn.close()
+        progress_bar()
         if len(self.expected_client_subjects) > 0:
             logger.info('Checking client certificate')
             self.client_certificate = load_certificate(FILETYPE_PEM, Path(self.client_pem_path).read_bytes())
@@ -160,7 +169,7 @@ class Transport:
                 logger.debug(f'expected subject: {check.commonName}')
                 if self.client_certificate.get_issuer().commonName == check.commonName:
                     self.client_certificate_match = True
-                    if isinstance(progress, Progress): progress.update(task, advance=1)
+                    progress_bar()
                     return True
         return False
 
@@ -241,7 +250,7 @@ class Transport:
             if proceed:
                 try:
                     conn.setblocking(0)
-                    conn.do_handshake()
+                    util.do_handshake(conn)
                     self.client_renegotiation = conn.total_renegotiations() > total_renegotiations
                 except SSL.ZeroReturnError: pass
                 except Exception as ex:
@@ -330,6 +339,11 @@ class Transport:
             self.ocsp_certificate_status = util.OCSP_CERT_STATUS[response.certificate_status.value]
         return response.response_status == OCSPResponseStatus.SUCCESSFUL and response.certificate_status == OCSPCertStatus.GOOD
 
+    def prepare_socket(self, timeout :int = 1):
+        sock = socket(AF_INET, SOCK_STREAM)
+        sock.settimeout(timeout)
+        return sock
+
     def prepare_context(self, method :str = None, verify_mode :str = None, check_hostname :bool = False) -> SSL.Context:
         if not isinstance(check_hostname, bool):
             raise TypeError(f'check_hostname {type(check_hostname)}, bool supported')
@@ -355,17 +369,17 @@ class Transport:
         ctx.check_hostname = check_hostname
         return ctx
 
-    def prepare_connection(self, context :SSL.Context) -> SSL.Connection:
-        ctx = ssl.SSLContext()
+    def prepare_connection(self, context :SSL.Context, sock :socket = None, use_sni :bool = True, protocol :int = None) -> SSL.Connection:
+        ctx = ssl.SSLContext() if protocol is None else ssl.SSLContext(protocol=protocol)
         ctx.verify_mode = ssl.CERT_NONE
         ctx.check_hostname = False
-        sock = socket(AF_INET, SOCK_STREAM)
-        sock.settimeout(1)
+        if sock is None:
+            sock = self.prepare_socket()
         conn = SSL.Connection(context, ctx.wrap_socket(sock, do_handshake_on_connect=False))
         conn.connect((self.host, self.port))
         conn.set_connect_state()
-        if ssl.HAS_SNI is True:
-            conn.set_tlsext_host_name(idna.encode(self.host))        
+        if all([ssl.HAS_SNI, use_sni]):
+            conn.set_tlsext_host_name(idna.encode(self.host))
         conn.setblocking(1)
         return conn
 
@@ -378,12 +392,14 @@ class Transport:
             f = None
         return f
 
-    @retry((SSL.WantReadError), tries=20, delay=0.5, logger=logger)
     def _http2(self, conn :SSL.Connection, request :Frame, response_wait :int = 3) -> Frame:
         logger.info(f'HTTP/2 Request:\n{request}')
         try:
             conn.sendall(b'PRI * HTTP/2.0\r\n\r\nSM\r\n\r\n')
             conn.sendall(request.serialize())
+        except SSL.WantReadError:
+            sleep(0.5)
+            return self._http2(conn, request, response_wait)
         except SSL.Error as err:
             if 'protocol is shutdown' not in str(err):
                 logger.warning(err, exc_info=True)
@@ -398,7 +414,7 @@ class Transport:
                 data = conn.recv(1024)
                 if not data: break
                 frame_data += data
-            except (SSL.ZeroReturnError, SSL.WantReadError):
+            except (SSL.ZeroReturnError):
                 break
             except Exception as ex:
                 logger.warning(ex, exc_info=True)
@@ -413,7 +429,7 @@ class Transport:
             conn.close()
         return resp
 
-    def test_h2c(self, method :str = 'GET', uri_path :str = '/', timeout :int = 3, include_settings :bool = True) -> Frame:
+    def test_h2c(self, method :str = 'GET', uri_path :str = '/', response_wait :int = 3, include_settings :bool = True) -> Frame:
         logger.info(f'Testing HTTP/2 clear text')
         def _read(s :socket):
             resp = b''
@@ -447,7 +463,7 @@ class Transport:
         request = '\r\n'.join(req_parts) + '\r\n\r\n'
         self.http2_cleartext_support = False
         sock = socket(AF_INET, SOCK_STREAM)
-        sock.settimeout(timeout)
+        sock.settimeout(response_wait)
         try:
             sock.connect((self.host, 80 if self.port == 443 else self.port))            
             sock.sendall(preamble)
@@ -477,56 +493,70 @@ class Transport:
             raise AttributeError(f'uri_path not supported')
         if not uri_path.startswith('/'):
             uri_path = f'/{uri_path}'
-        def _con():
-            ctx = self.prepare_context()
-            ctx.set_alpn_protos([b'h2', b'http/1.1'])
-            # rfc7540 section 9.2
-            ctx.set_options(ssl.OP_NO_COMPRESSION | ssl.OP_NO_SSLv2 | ssl.OP_NO_SSLv3 | ssl.OP_NO_TLSv1 | ssl.OP_NO_TLSv1_1 | ssl.OP_NO_TLSv1_3)
-            ctx.set_cipher_list(b"ECDHE+AESGCM:ECDHE+CHACHA20:DHE+AESGCM:DHE+CHACHA20")
-            conn = self.prepare_connection(ctx)
-            conn.settimeout(response_wait)
-            try:
-                conn.do_handshake()
-            except Exception as ex:
-                logger.warning(ex, exc_info=True)
-            return conn
-
+        ctx = self.prepare_context()
+        ctx.set_alpn_protos([b'h2', b'http/1.1'])
+        # rfc7540 section 9.2
+        ctx.set_options(ssl.OP_NO_COMPRESSION | ssl.OP_NO_SSLv2 | ssl.OP_NO_SSLv3 | ssl.OP_NO_TLSv1 | ssl.OP_NO_TLSv1_1 | ssl.OP_NO_TLSv1_3)
+        ctx.set_cipher_list(b"ECDHE+AESGCM:ECDHE+CHACHA20:DHE+AESGCM:DHE+CHACHA20")
+        conn = self.prepare_connection(context=ctx, sock=self.prepare_socket(response_wait))
+        conn.settimeout(response_wait)
+        util.do_handshake(conn)
         request = Transport.decode_http2_frame(b'\x00\x00\x13\x00\x09\x00\x00\x00\x01\x0Ahello' + b'\0' * 10)
-        resp = self._http2(_con(), request, response_wait)
+        resp = self._http2(conn, request, response_wait)
         if isinstance(resp, Frame):
             self.http2_support = True
             self.http2_response_frame = str(resp)
 
+    def test_highest_tls_version(self, version, response_wait :int = 3):
+        logger.warning(DeprecationWarning('Transport.test_highest_tls_version() was deprecated in version 0.4.4 and will be removed in version 0.5.0'), exc_info=True)
+        self.test_tls_version(min_tls_version=version, response_wait=response_wait)
+
+    def test_tls_version(self, min_tls_version :int = None, max_tls_version :int = None, use_sni :bool = False, response_wait :int = 3) -> str:
+        if min_tls_version is not None and not isinstance(min_tls_version, int):
+            raise TypeError(f'min_tls_version {type(min_tls_version)}, int supported')
+        if max_tls_version is not None and not isinstance(max_tls_version, int):
+            raise TypeError(f'max_tls_version {type(max_tls_version)}, int supported')
+        if not isinstance(response_wait, int):
+            raise TypeError(f'response_wait {type(response_wait)}, int supported')
+        protocol = None
+        ctx = self.prepare_context()
+        ctx.set_options(_util.lib.SSL_OP_TLS_ROLLBACK_BUG)
+        if min_tls_version is not None:
+            logger.info(f'min protocol {util.OPENSSL_VERSION_LOOKUP[min_tls_version]}')
+            ctx.set_min_proto_version(min_tls_version)
+        if max_tls_version is not None:
+            logger.info(f'max protocol {util.OPENSSL_VERSION_LOOKUP[max_tls_version]}')
+            ctx.set_max_proto_version(max_tls_version)
+        conn = self.prepare_connection(context=ctx, sock=self.prepare_socket(timeout=response_wait), use_sni=use_sni)
+        conn.settimeout(response_wait)
+        util.do_handshake(conn)
+        protocol = conn.get_protocol_version_name()
+        logger.info(f'Negotiated {protocol}')
+        conn.shutdown()
+        conn.close()
+        return protocol
+
     def connect(self, tls_version :int, use_sni :bool = False, protocol :str = None):
         logger.info(f'Trying TLS version {util.OPENSSL_VERSION_LOOKUP[tls_version]}')
-        ctx = SSL.Context(method=getattr(SSL, Transport.default_connect_method))
-        ctx.load_verify_locations(cafile=where())
-        for cafile in self.cafiles:
-            ctx.load_verify_locations(cafile=cafile)
-        if self.client_pem_path is not None:
-            ctx.use_certificate_file(certfile=self.client_pem_path, filetype=FILETYPE_PEM)
+        ctx = self.prepare_context()
         ctx.set_verify(getattr(SSL, Transport.default_connect_verify_mode), self._verifier)
         ctx.set_max_proto_version(tls_version)
-        ctx.check_hostname = False
-        sock = socket(AF_INET, SOCK_STREAM)
-        sock.settimeout(1)
-        conn = SSL.Connection(context=ctx, socket=sock)
-
-        conn.set_connect_state()
         ctx.set_ocsp_client_callback(self._ocsp_handler)
+        ctx.set_options(_util.lib.SSL_OP_ALLOW_UNSAFE_LEGACY_RENEGOTIATION | _util.lib.SSL_OP_LEGACY_SERVER_CONNECT)
+        conn = SSL.Connection(context=ctx, socket=self.prepare_socket())
         conn.request_ocsp()
         if all([use_sni, ssl.HAS_SNI]):
             logger.info('using SNI')
-            conn.set_tlsext_host_name(idna.encode(self.host))        
+            conn.set_tlsext_host_name(idna.encode(self.host))
         try:
-            logger.info('connect')
             conn.connect((self.host, self.port))
+            conn.set_connect_state()
             conn.setblocking(1)
-            logger.info('handshake')
-            conn.do_handshake()
-            logger.info('read')
+            util.do_handshake(conn)
             self.negotiated_cipher = conn.get_cipher_name()
-            self.negotiated_protocol = conn.get_protocol_version_name()
+            negotiated_protocol = conn.get_protocol_version_name()
+            self.negotiated_protocol = f'{negotiated_protocol} ({hex(util.PROTOCOL_VERSION[negotiated_protocol])})'
+            self.offered_tls_versions.append(self.negotiated_protocol)
             self.session_cache_mode = util.SESSION_CACHE_MODE[native_openssl.SSL_CTX_get_session_cache_mode(conn._context._context)]
             self.session_tickets = native_openssl.SSL_SESSION_has_ticket(conn.get_session()._session) == 1
             self.session_ticket_hints = native_openssl.SSL_SESSION_get_ticket_lifetime_hint(conn.get_session()._session) > 0
@@ -549,42 +579,134 @@ class Transport:
         finally:
             conn.close()
 
-    def connect_least_secure(self, cafiles :list = None, use_sni :bool = False, updater :tuple[Progress, TaskID] = None) -> bool:
+    def test_scsv(self, tls_version :int, use_sni :bool = True):
+        """
+        A connection of higher protocol indicates we have previously connected using the client fallback mechanism
+        When a handshake is rejected (False) assume downgrade attacks were prevented
+        """
+        logger.info('Trying to derive SCSV')
+        try:
+            negotiated = self.test_tls_version(min_tls_version=tls_version, use_sni=use_sni)
+            if negotiated:    
+                self.preferred_protocol = f'{negotiated} ({hex(util.PROTOCOL_VERSION[negotiated])})'
+            else:
+                self.preferred_protocol = self.negotiated_protocol
+            self.offered_tls_versions.append(self.preferred_protocol)
+            self.tls_downgrade = negotiated is not None
+        except Exception:
+            self.tls_downgrade = False
+
+    def test_tls_all_versions(self, use_sni :bool = True):
+        logger.info('Testing all TLS versions')
+        for ver_name, tls_version in util.PROTOCOL_VERSION.items():
+            ver_display_name = f'{ver_name} ({hex(tls_version)})'
+            if ver_name in FAKE_PROTOCOLS or ver_display_name in self.offered_tls_versions:
+                continue
+            try:
+                negotiated = self.test_tls_version(max_tls_version=tls_version, use_sni=use_sni)
+                supported = negotiated == ver_name
+            except Exception:
+                supported = False
+            if supported is True:
+                self.offered_tls_versions.append(ver_display_name)
+
+    def test_tls_version_interference(self):
+        """
+        A rejected connection (typically the oldest or latest version, currently 1.3)
+        when no mutual accepted TLS version can be negotiates is known as tls interference
+        """
+        for check_interference in ['TLSv1.3 (0x304)', 'TLSv1.2 (0x303)', 'TLSv1.1 (0x302)', 'TLSv1 (0x301)', 'SSLv3 (0x300)', 'SSLv2 (0x2ff)']:
+            if check_interference not in self.offered_tls_versions:
+                self.tls_version_interference_versions.append(check_interference)
+        self.tls_version_interference = len(self.tls_version_interference_versions) > 0
+
+    def test_tls_version_intolerance(self, use_sni :bool = True):
+        """
+        Protocol not understood by the server, the server should negotiate the highest protocol it knows
+        A rejected connection indicates TLS version intolerance, and is not rfc5246 or rfc8446 compliant
+        """
+        logger.info('Trying to derive TLS version intolerance')
+        for fake_proto in FAKE_PROTOCOLS:
+            fake_ver = util.PROTOCOL_VERSION[fake_proto]
+            try:
+                intolerance = self.test_tls_version(min_tls_version=fake_ver, use_sni=use_sni) is None
+                if intolerance:
+                    self.tls_version_intolerance_versions.append(f'{fake_proto} ({hex(fake_ver)})')
+            except Exception:
+                self.tls_version_intolerance_versions.append(f'{fake_proto} ({hex(fake_ver)})')
+        self.tls_version_intolerance = len(self.tls_version_intolerance_versions) > 0
+        self.tls_version_interference = False
+
+    def connect_least_secure(self, cafiles :list = None, use_sni :bool = False, progress_bar :callable = lambda *args: None) -> bool:
         if not isinstance(self.port, int):
             raise TypeError(f"provided an invalid type {type(self.port)} for port, expected int")
         if validators.domain(self.host) is not True:
             raise ValueError(f"provided an invalid domain {self.host}")
-        progress, task = (None, None)
-        if isinstance(updater, tuple):
-            progress, task = updater
         if cafiles is not None:
             if not isinstance(cafiles, list):
                 raise TypeError(f"provided an invalid type {type(cafiles)} for cafiles, expected list")
             valid_cafiles = util.filter_valid_files_urls(cafiles)
             if valid_cafiles is False:
-                raise AttributeError(f'cafiles was provided but is not a valid URLs or files do not exist')
-            if isinstance(valid_cafiles, list):
-                self.cafiles = valid_cafiles
-        for version in [SSL.SSL3_VERSION, SSL.TLS1_VERSION, SSL.TLS1_1_VERSION, SSL.TLS1_2_VERSION, SSL.TLS1_3_VERSION]:
-            self.connect(tls_version=version, use_sni=use_sni)
-            if isinstance(progress, Progress): progress.update(task, advance=1)
-            if isinstance(self.server_certificate, X509):
-                if all([use_sni, ssl.HAS_SNI]):
-                    self.sni_support = True
-                for protocol in ['HTTP/1.0', 'HTTP/1.1']:
-                    self.connect(tls_version=version, use_sni=use_sni, protocol=protocol)
-                self.test_h2c()
-                self.test_http2()
-                if isinstance(progress, Progress): progress.update(task, advance=1)
+                raise AttributeError('cafiles was provided but is not a valid URLs or files do not exist')
+            if isinstance(valid_cafiles, list): self.cafiles = valid_cafiles
+
+        tls_versions = [SSL.SSL3_VERSION, SSL.TLS1_VERSION, SSL.TLS1_1_VERSION, SSL.TLS1_2_VERSION, SSL.TLS1_3_VERSION]
+        for index, version in enumerate(tls_versions):
+            self.connect(tls_version=version, use_sni=use_sni) # Skip HTTP testing until negotiated
+            progress_bar()
+            if not isinstance(self.server_certificate, X509):
+                continue
+            progress_bar(5)
+
+            if all([use_sni, ssl.HAS_SNI]):
+                self.sni_support = True
+            for protocol in ['HTTP/1.0', 'HTTP/1.1']:
+                self.connect(tls_version=version, use_sni=use_sni, protocol=protocol)
+                progress_bar()
+
+            self.test_h2c(response_wait=3)
+            progress_bar()
+
+            self.test_http2(response_wait=3)
+            progress_bar()
+
+            if version == SSL.TLS1_3_VERSION:
+                # Already the highest TLS protocol, no downgrade possible
+                self.tls_downgrade = False
+                # server can only prefer this too
+                self.preferred_protocol = util.OPENSSL_VERSION_LOOKUP[version]
+                self.test_tls_version_interference()
+                progress_bar(12)
+                self.test_tls_version_intolerance(use_sni) # sourcery skip: extract-duplicate-method
+                progress_bar()
                 return True
+
+            self.test_scsv(tls_versions[index+1], use_sni)
+            progress_bar()
+            
+            self.test_tls_all_versions(use_sni)
+            progress_bar()
+
+            self.test_tls_version_interference()
+            progress_bar()
+
+            self.test_tls_version_intolerance(use_sni)
+            progress_bar()
+
+            return True
+
         return False
 
     @staticmethod
-    def is_connection_closed(conn: SSL.Connection) -> bool:
+    def is_connection_closed(conn: SSL.Connection, counter :int = 0, max_retries :int = 5) -> bool:
         try:
             data = conn.recv(1, MSG_PEEK)
             if len(data) == 0:
                 return True
+        except SSL.WantReadError:
+            if counter >= max_retries: return True
+            sleep(0.5)
+            return Transport.is_connection_closed(conn, counter+1)
         except BlockingIOError:
             return False
         except SSL.ZeroReturnError:
